local Tablet = AceLibrary("Tablet-2.0")
Click2Cast = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

Click2Cast.version = "r" .. string.sub("$Revision: 1 $", 12, -3)
Click2Cast.date = string.sub("$Date: 2006-12-24 04:03:00 +0200 (Sun, 22 Oct 2006) $", 8, 17)
Click2Cast.hasIcon = "Interface\\CURSOR\\Cast"
Click2Cast.defaultPosition = 'RIGHT'
Click2Cast.hideWithoutStandby = true
Click2Cast.independentProfile = true
Click2Cast.actionText = {["macro"] = true,["spell"] = true}

function Click2Cast:OnInitialize()
	Click2Cast:RegisterDB("Click2CastDB")
	Click2Cast:RegisterDefaults('profile', {   
        clickOptions = {
            ['*'] = { action = "none" },
            ["BUTTON1"] = { action = "target" },
            ["BUTTON2"] = { action = "menu" },
        }
    })
	Click2Cast:CreateMenu()
end

function Click2Cast:OnEnable()
	if not ClickCastFrames then
		ClickCastFrames = {}
	end
	self.clickFrames = ClickCastFrames
	ClickCastFrames[PlayerFrame] = true
	ClickCastFrames[TargetFrame] = true
	setmetatable(self.clickFrames, { __newindex = function(self,k,v)
		if v == nil then
            Click2Cast:UnregisterFrame(k)
		else
            Click2Cast:RegisterFrame(k)
		end
		rawset(self, k, v)
	end })
	self:SetAllFrames()
end

function Click2Cast:OnProfileEnable()
	self:RecreateMenu()
	self:SetAllFrames()
end
    
function Click2Cast:SetAllFrames()
    if InCombatLockdown() and not self.OOCSetAllFrames then
        self:ScheduleLeaveCombatAction("SetAllFrames")
        self.OOCSetAllFrames = true        
        return 
    end    
	for frame in pairs(self.clickFrames) do
		self:RegisterFrame(frame)
	end
    self.OOCSetAllFrames = nil    
end

function Click2Cast:OnDisable()
    if InCombatLockdown() and not self.OOCOnDisable then
        self:ScheduleLeaveCombatAction("OnDisable")
        self.OOCOnDisable = true
        return 
    end
	for frame in pairs(self.clickFrames) do
		self:UnregisterFrame(frame)
	end
    self.OOCOnDisable = nil
end

function Click2Cast:RegisterFrame(frame)
	frame:RegisterForClicks("LeftButtonUp", "MiddleButtonUp", "RightButtonUp", "Button4Up", "Button5Up")
	for k,v in pairs(self.db.profile.clickOptions) do
        if type(k) == "string" then
            local button = k:match("(%d+)")
            local modifier = k:gsub("BUTTON%d", "")
            if self.actionText[v.action] then
                self:SetAttribute(frame, modifier .. "type" .. button, v.action)
                local actionText,lAction
                if v.action == "spell" and v.rank and string.find(v.rank,"(%d+)") then
                    actionText = string.format(v.rank and "%s(Rank %s)" or "%s", v.actiontext, v.rank)
                elseif v.action == "macro" then
                    lAction = "macrotext"
                end
                self:SetAttribute(frame, modifier .. (lAction or v.action) .. button, actionText or v.actiontext)
            else
                self:SetAttribute(frame, modifier .. "type" .. button, v.action)		
            end
        end
	end
end

function Click2Cast:UnregisterFrame(frame)
	for k,v in pairs(self.db.profile.clickOptions) do
        local button = k:match("(%d+)")
        local modifier = k:gsub("BUTTON%d", "")
        if modifier == "None-" then
            modifier = ""
        end
        self:SetAttribute(frame, modifier .. "type" .. button, nil)
        self:SetAttribute(frame, modifier .. v.action .. button, nil)   
	end
end

function Click2Cast:SetAttribute(frame,key,value)
    if InCombatLockdown() then 
        self:ScheduleLeaveCombatAction(frame.SetAttribute,frame,key,value)
        return 
    end
    frame:SetAttribute(key,value)
end

