local dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Click2Cast")
local T = AceLibrary("Tablet-2.0")

local actions = {
	menu = L["menu"],
	spell = L["spell"],
	target = L["target"],
	focus = L["focus"],
	assist = L["assist"],
	macro = L["macro"],
	none = L["none"],
}

local keybindingOnly = {BUTTON1=true, BUTTON2=true, BUTTON3=true, BUTTON4=true, BUTTON5=true}

local function getOrder(binding)
	local num = 0
	if binding:find("ALT%-") then
		num = num + 1
	end
	if binding:find("CTRL%-") then
		num = num + 2
	end
	if binding:find("SHIFT%-") then
		num = num + 4
	end
	
	local button = tonumber(binding:match("BUTTON(%d)$"))
	num = num + button*8
	return num
end

local function createSubMenu(self, value)
	local name = dewdrop:EncodeKeybinding(value)
	self.optionsTable.args[value] = {
		name = name or value.."bug",
		desc = name or value.."bug",
		order = getOrder(value),
		type = 'group',
		args = {
			actions = {
				type = 'text',
				name = L["Set Action"],
				desc = L["actions"],
				order = 1,
				validate = actions,
				get = function() 
					return self.db.profile.clickOptions[value].action
				end,
				set = function(option)
					self.db.profile.clickOptions[value].action = option
					self:SetAllFrames()
					if self.actionText[self.db.profile.clickOptions[value].action] then
						self.optionsTable.args[value].args.action.name = L[option]
						self.optionsTable.args[value].args.action.desc = L[option.."Desc"]
                        local parent = dewdrop:GetOpenedParent()
                        if parent:GetName() == "FuBarPluginClick2CastFrame" then
                            dewdrop:Refresh(2)
                            dewdrop:Close(2)
                        else
                            dewdrop:Refresh(3)
                            dewdrop:Close(3)
                        end                        
					end
				end,
			},
			remove = {
				type = 'execute',
				name = L["Remove"],
				desc = L["RemoveDesc"],
				func = function()
					self.db.profile.clickOptions[value] = nil
					self.optionsTable.args[value] = nil
					self:SetAllFrames()
                    local parent = dewdrop:GetOpenedParent()
                    if parent:GetName() == "FuBarPluginClick2CastFrame" then
                        dewdrop:Refresh(1)
                        dewdrop:Close(2)
                    else
                        dewdrop:Refresh(2)
                        dewdrop:Close(3)
                    end
				end,
			},
			action = {
				name = L["Action Text"],
				type = 'text',
				desc = L["actiontext"],
				order = 2,
				get = function()
					return self.db.profile.clickOptions[value].actiontext or ""
				end,
				usage = "",
				set = function(option)
					self.db.profile.clickOptions[value].actiontext = option
					self:SetAllFrames()
				end,
				hidden = function()
					return not (self.actionText[self.db.profile.clickOptions[value].action])
				end,
			},
			rank = {
				name = L["Rank"],
				type = 'text',
				desc = L["rankDesc"],
				order = 3,
				get = function()
					return self.db.profile.clickOptions[value].rank or ""
				end,
				usage = "",
				set = function(option)
					self.db.profile.clickOptions[value].rank = option:match("(%d+)")
					self:SetAllFrames()
				end,
				hidden = function()
					return not (self.db.profile.clickOptions[value].action == "spell")
				end,
			}			
		}
	}
	if self.actionText[self.db.profile.clickOptions[value].action] then
		self.optionsTable.args[value].args.action.name = L[self.db.profile.clickOptions[value].action]
		self.optionsTable.args[value].args.action.desc = L[self.db.profile.clickOptions[value].action.."Desc"]
	end
end

function Click2Cast:CreateMenu()
	self.optionsTable = {
		name = L["C2C"],
		desc = L["C2CDesc"],
        order = 3,
		handler = Click2Cast,
		type = 'group',
		args = {},
		disabled = InCombatLockdown,
	}	
	self.optionsTable.args.new = {
		name = "New binding",
		desc = "New binding",
		order = 200,
		type = 'text',
		validate = "keybinding",
		set = function(value)
			if not self.optionsTable.args[value] then
				createSubMenu(self, value)
			end
		end,
		get = function()
			return nil
		end,
		keybindingOnly = keybindingOnly,
	}
	for k in pairs(self.db.profile.clickOptions) do
		createSubMenu(self, k)
	end
	
	Click2Cast:RegisterChatCommand({ "/Click2Cast", "/c2c", "/cc" }, self.optionsTable)
	Click2Cast.OnMenuRequest = self.optionsTable
end

local tmp = {}
local function mysort(alpha, bravo)
	if not alpha or not bravo then
		return false
	end
	return getOrder(alpha) < getOrder(bravo)
end

function Click2Cast:OnTooltipUpdate()
	local db = self.db.profile.clickOptions
	local cat
	
	local showAction = {["spell"] = true,}
	for k,v in pairs(db) do
		table.insert(tmp, k)
	end
	table.sort(tmp, mysort)
	local last_button = nil
	for i,k in ipairs(tmp) do
		tmp[i] = nil
		local v = db[k]
		local button = tonumber(k:match("BUTTON(%d)$"))
		if last_button ~= button then
			last_button = button
			cat = T:AddCategory(
				'text', dewdrop:EncodeKeybinding("BUTTON" .. button),
				'columns', 2,
				'child_textR', 1,
				'child_textG', 1,
				'child_textB', 0,
				'child_textR2', 1,
				'child_textG2', 1,
				'child_textB2', 1
			)
		end
		local noMod
		if not k:find("%-") then
			noMod = L["No Modifier"]
		end
		if v.actiontext and showAction[v.action] then
			cat:AddLine(
				'text', noMod or dewdrop:EncodeKeybinding(k),
				'text2', v.actiontext
			)
		else
			cat:AddLine(
				'text', noMod or dewdrop:EncodeKeybinding(k),
				'text2', L[v.action]
			)
		end
	end
	
	T:SetHint(L["Right-click to configure clickcast actions."])
end

function Click2Cast:RecreateMenu()
	for k in pairs(self.optionsTable.args) do
		if k:find("^ALT.-BUTTON%d$") or k:find("^CTRL.-BUTTON%d$") or k:find("^SHIFT.-BUTTON%d$") or k:find("^BUTTON%d$") then
			self.optionsTable.args[k] = nil
		end
	end
	for k in pairs(self.db.profile.clickOptions) do
		createSubMenu(self, k)
	end
end
